@@ -1,7 +1,7 @@
 package Getopt::Long::Subcommand;
 
-our $DATE = '2017-01-03'; # DATE
-our $VERSION = '0.07'; # VERSION
+our $DATE = '2017-01-06'; # DATE
+our $VERSION = '0.08'; # VERSION
 
 use 5.010001;
 use strict;
@@ -48,7 +48,7 @@ sub _cmdspec_opts_to_gl_ospec {
 sub _gl_getoptions {
     require Getopt::Long;
 
-    my ($ospec, $pass_through) = @_;
+    my ($ospec, $pass_through, $res) = @_;
     #$log->tracef('[comp][glsubc] Performing Getopt::Long::GetOptions');
 
     my $old_conf = Getopt::Long::Configure(
@@ -56,9 +56,22 @@ sub _gl_getoptions {
         ('pass_through') x !!$pass_through,
     );
     local $SIG{__WARN__} = sub {} if $pass_through;
+
+    # ugh, this is ugly. the problem we're trying to solve: in the case of 'subc
+    # --help', 'subc' is consumed first by Getopt::Long and thus removed from
+    # @ARGV. when --help handler wants to find out the subcommand name ('subc'),
+    # it doesn't have anywhere to look for. so we give it in $res which is
+    # passed as the third argument to the handler.
+    local $res->{_non_options_argv} = [];
+
     #$log->tracef('[comp][glsubc] @ARGV before Getopt::Long::GetOptions: %s', \@ARGV);
     #$log->tracef('[comp][glsubc] spec for Getopt::Long::GetOptions: %s', $ospec);
-    my $gl_res = Getopt::Long::GetOptions(%$ospec);
+    my $gl_res = Getopt::Long::GetOptions(
+        %$ospec,
+        '<>' => sub { push @{ $res->{_non_options_argv} }, $_[0] },
+    );
+    @ARGV = @{ $res->{_non_options_argv} };
+
     #$log->tracef('[comp][glsubc] @ARGV after  Getopt::Long::GetOptions: %s', \@ARGV);
     Getopt::Long::Configure($old_conf);
     $gl_res;
@@ -95,7 +108,7 @@ sub _GetOptions {
 
     my $ospec = _cmdspec_opts_to_gl_ospec(
         $cmdspec->{options}, $is_completion, $res);
-    unless (_gl_getoptions($ospec, $pass_through)) {
+    unless (_gl_getoptions($ospec, $pass_through, $res)) {
         $res->{success} = 0;
         return $res;
     }
@@ -116,10 +129,8 @@ sub _GetOptions {
         }
 
         unless (@ARGV) {
-            warn "Missing subcommand".
-                ($stash->{path} ? " for $stash->{path}":"")."\n"
-                    unless $is_completion;
-            $res->{success} = 0;
+            # no subcommand name
+            $res->{success} = 1;
             return $res;
         }
         my $sc_name = shift @ARGV;
@@ -250,7 +261,7 @@ Getopt::Long::Subcommand - Process command-line options, with subcommands and co
 
 =head1 VERSION
 
-This document describes version 0.07 of Getopt::Long::Subcommand (from Perl distribution Getopt-Long-Subcommand), released on 2017-01-03.
+This document describes version 0.08 of Getopt::Long::Subcommand (from Perl distribution Getopt-Long-Subcommand), released on 2017-01-06.
 
 =head1 SYNOPSIS
 
